﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using TreeViewMVVM.Web.Services;
using System.Windows.Navigation;
using System.ComponentModel;
using System.Linq.Expressions;
using TreeViewMVVM.Controls;
using System.Diagnostics;

namespace TreeViewMVVM
{
    public class BaseViewModel : DependencyObject, INotifyPropertyChanged //, IRaisePropertyChanged
    {
        #region Constructors
        protected BaseViewModel()
        {
            _currentContext = new TreeViewMVVMDomainContext();
        }
        #endregion

        #region INotifyPropertyChanged
        
        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        protected virtual void OnPropertyChanged<TProperty>(Expression<Func<TProperty>> property)
        {
            var lambda = (LambdaExpression)property;

            MemberExpression memberExpression;
            if (lambda.Body is UnaryExpression)
            {
                var unaryExpression = (UnaryExpression)lambda.Body;
                memberExpression = (MemberExpression)unaryExpression.Operand;
            }
            else memberExpression = (MemberExpression)lambda.Body;

            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(memberExpression.Member.Name));
            }
        }

        #endregion

        /*
        #region IRaisePropertyChanged
        void IRaisePropertyChanged.RaisePropertyChanged(string propertyName)
        {
            OnPropertyChanged(propertyName);
        }
        #endregion
        */

        #region Properties
        protected bool IsInDesignMode { get { return App.Current.RootVisual != null; } }
        #endregion

        #region Domain Context
        private TreeViewMVVMDomainContext _currentContext;
        public TreeViewMVVMDomainContext CurrentContext
        {
            get { return _currentContext; }
            set { _currentContext = value; }
        }
        #endregion

        #region Events
        public event EventHandler<NavigationEventArgs> Navigate;
        protected void OnNavigate(NavigationEventArgs e)
        {
            if (Navigate != null)
            {
                Navigate(this, e);
            }
        }
        #endregion

        #region Dialog Support
        protected void ShowMessageWindow(string message)
        {
            ShowMessageWindow(message, String.Empty, MessageBoxButtons.OK, null);
        }

        protected void ShowMessageWindow(string message, Action<MessageBoxResult> callback)
        {
            ShowMessageWindow(message, String.Empty, MessageBoxButtons.OK, callback);
        }

        protected void ShowMessageWindow(string message, MessageBoxButtons buttons, Action<MessageBoxResult> callback)
        {
            ShowMessageWindow(message, String.Empty, buttons, callback);
        }

        protected void ShowMessageWindow(string message, string title, Action<MessageBoxResult> callback)
        {
            ShowMessageWindow(message, title, MessageBoxButtons.OK, callback);
        }

        protected void ShowMessageWindow(string message, string title, MessageBoxButtons buttons, Action<MessageBoxResult> callback)
        {
            MessageWindowEventArgs args = new MessageWindowEventArgs()
            {
                Title = title,
                Message = message,
                Buttons = buttons
            };

            MessageWindow.ShowMessageWindow(args, callback);
        }
        #endregion

    }
}
