﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace TreeViewMVVM.Controls
{
    public partial class MessageWindow : ChildWindow
    {
        public MessageWindow()
        {
            InitializeComponent();
        }

        public static void ShowMessageWindow(MessageWindowEventArgs args, Action<MessageBoxResult> callback)
        {
            MessageWindowViewModel vm = new MessageWindowViewModel() { Arguments = args };
            MessageWindow msgWnd = new MessageWindow() { DataContext = vm };

            vm.CloseWindow += (s, e) => { msgWnd.Close(); };
            msgWnd.Closed += (s, e) => { if (callback != null) callback(args.Result); };
            msgWnd.Show();
        }
    }

    public enum MessageBoxButtons
    {
        OK,
        OKCancel,
        YesNo,
        YesNoCancel
    }

    public class MessageWindowEventArgs : EventArgs
    {
        public MessageWindowEventArgs()
        {

        }

        public string Title { get; set; }
        public string Message { get; set; }
        public MessageBoxButtons Buttons { get; set; }
        public MessageBoxResult Result { get; set; }
    }

    public class MessageWindowViewModel : BaseViewModel
    {
        public MessageWindowViewModel()
        {
            Arguments = new MessageWindowEventArgs();
        }

        private MessageWindowEventArgs _arguments;
        public MessageWindowEventArgs Arguments
        {
            get { return _arguments; }
            set
            {
                _arguments = value;
                NotifyAllChanged();
            }
        }

        public event EventHandler CloseWindow;
        protected void OnCloseWindow(EventArgs e)
        {
            if (CloseWindow != null)
            {
                CloseWindow(this, e);
            }
        }

        #region Show Properties
        public bool ShowOKButton
        {
            get
            {
                return _arguments == null ? false : (_arguments.Buttons == MessageBoxButtons.OK || _arguments.Buttons == MessageBoxButtons.OKCancel);
            }
        }

        public bool ShowCancelButton
        {
            get
            {
                return _arguments == null ? false : (_arguments.Buttons == MessageBoxButtons.OKCancel || _arguments.Buttons == MessageBoxButtons.YesNoCancel);
            }
        }

        public bool ShowYesButton
        {
            get
            {
                return _arguments == null ? false : (_arguments.Buttons == MessageBoxButtons.YesNo || _arguments.Buttons == MessageBoxButtons.YesNoCancel);
            }
        }

        public bool ShowNoButton
        {
            get
            {
                return _arguments == null ? false : (_arguments.Buttons == MessageBoxButtons.YesNo || _arguments.Buttons == MessageBoxButtons.YesNoCancel);
            }
        }
        #endregion

        #region Result Methods
        public void WindowClosing()
        {
            // force negative result if none was given
            if (Arguments.Result == MessageBoxResult.None)
            {
                switch (Arguments.Buttons)
                {
                    case MessageBoxButtons.OK:
                        Arguments.Result = MessageBoxResult.OK;
                        break;

                    case MessageBoxButtons.OKCancel:
                        Arguments.Result = MessageBoxResult.Cancel;
                        break;

                    case MessageBoxButtons.YesNo:
                        Arguments.Result = MessageBoxResult.No;
                        break;

                    case MessageBoxButtons.YesNoCancel:
                        Arguments.Result = MessageBoxResult.Cancel;
                        break;
                }
            }
        }

        public void OKClicked()
        {
            Arguments.Result = MessageBoxResult.OK;
            OnCloseWindow(EventArgs.Empty);
        }

        public void CancelClicked()
        {
            Arguments.Result = MessageBoxResult.Cancel;
            OnCloseWindow(EventArgs.Empty);
        }

        public void YesClicked()
        {
            Arguments.Result = MessageBoxResult.Yes;
            OnCloseWindow(EventArgs.Empty);
        }

        public void NoClicked()
        {
            Arguments.Result = MessageBoxResult.No;
            OnCloseWindow(EventArgs.Empty);
        }

        #endregion

        private void NotifyAllChanged()
        {
            OnPropertyChanged(() => Arguments);
            OnPropertyChanged(() => ShowOKButton);
            OnPropertyChanged(() => ShowCancelButton);
            OnPropertyChanged(() => ShowYesButton);
            OnPropertyChanged(() => ShowNoButton);
        }
    }

}

